@@ -8,34 +8,46 @@
     :copyright: Copyright 2013 Tony Narlock.
     :license: BSD, see LICENSE for details
 """
+from __future__ import absolute_import, division, print_function, with_statement
+
 import os
-from .util import tmux
+from .util import tmux, TmuxRelationalObject
 from .session import Session
 from .formats import SESSION_FORMATS, CLIENT_FORMATS
-from .logxtreme import logging
-from .exc import TmuxNotRunning, TmuxSessionExists
+
+from . import log
+import logging
+
+logger = logging.getLogger(__name__)
 
 
-class Server(object):
+class Server(TmuxRelationalObject):
 
     '''
     The :term:`tmux(1)` server. Container for:
 
-    :attr:`Server._sessions` [:class:`Session`, ..]
-        :attr:`Session._windows` [:class:`Window`, ..]
-            :attr:`Window._panes` [:class:`Pane`, ..]
-                :class:`Pane`
+    - :attr:`Server._sessions` [:class:`Session`, ...]
 
-    When instantiated, provides the``t`` global. stores information on live,
+      - :attr:`Session._windows` [:class:`Window`, ...]
+
+        - :attr:`Window._panes` [:class:`Pane`, ...]
+
+          - :class:`Pane`
+
+    When instantiated, provides the ``t`` global. stores information on live,
     running tmux server.
     '''
 
     socket_name = None
     socket_path = None
     config_file = None
+    childIdAttribute = 'session_id'
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         self._sessions = list()
+        self.children = self._sessions
+
+        self._TMUX = {}
 
     def tmux(self, *args, **kwargs):
         args = list(args)
@@ -47,22 +59,6 @@ class Server(object):
             args.insert(0, '-f{}'.format(self.config_file))
         return tmux(*args, **kwargs)
 
-    def hotswap(self, session_name=None):
-        args = ['/usr/local/bin/tmux', 'tmux']
-        if self.socket_name:
-            args.append('-L{}'.format(self.socket_name))
-        if self.socket_path:
-            args.append('-S{}'.format(self.socket_path))
-        if self.config_file:
-            args.append('-f{}'.format(self.config_file))
-        args.append('attach-session')
-        if session_name:
-            args.append('-t{}'.format(session_name))
-
-        #logging.info(args)
-        os.execl(*args)
-        #os.execl('/usr/local/bin/tmux', 'tmux', 'attach-session', '-t', session_name)
-
     def list_sessions(self):
         '''
         Return a list of :class:`Session` from tmux server.
@@ -74,11 +70,11 @@ class Server(object):
         sessions = self.tmux(
             'list-sessions',
             '-F%s' % '\t'.join(tmux_formats),   # output
-            _iter=False                          # iterate line by line
-        )
+        ).stdout
 
         # combine format keys with values returned from ``tmux list-windows``
-        sessions = [dict(zip(formats, session.split('\t'))) for session in sessions]
+        sessions = [dict(zip(
+            formats, session.split('\t'))) for session in sessions]
 
         # clear up empty dict
         new_sessions = [
@@ -87,19 +83,21 @@ class Server(object):
 
         if not self._sessions:
             for session in new_sessions:
-                logging.debug('adding session_id %s' % (session['session_id']))
+                logger.debug('adding session_id %s' % (session['session_id']))
                 new_session = Session(server=self, **session)
                 self._sessions.append(new_session)
             return self._sessions
 
         new = {session['session_id']: session for session in new_sessions}
-        old = {session.get('session_id'): session for session in self._sessions}
+        old = {session.get(
+            'session_id'): session for session in self._sessions}
 
         created = set(new.keys()) - set(old.keys()) or ()
         deleted = set(old.keys()) - set(new.keys()) or ()
         intersect = set(new.keys()).intersection(set(old.keys()))
 
-        diff = {id: dict(set(new[id].items()) - set(old[id].items())) for id in intersect}
+        diff = {id: dict(set(new[id].items()) - set(old[id].items()))
+                for id in intersect}
 
         intersect = set(k for k, v in diff.iteritems() if v) or ()
         diff = dict((k, v) for k, v in diff.iteritems() if v) or ()
@@ -115,25 +113,27 @@ class Server(object):
         if deleted:
             log_diff += "deleted %s" % deleted
         if log_diff:
-            logging.info(log_diff)
+            logger.debug(log_diff)
 
         for s in self._sessions:
             # remove session objects if deleted or out of session
             if s.get('session_id') in deleted:
-                logging.info("removing %s" % s)
+                logger.debug("removing %s" % s)
                 self._sessions.remove(s)
 
             if s.get('session_id') in intersect and s.get('session_id') in diff:
-                logging.debug('updating session_id %s session_name %s' % (s.get('session_id'), s.get('session_name')))
+                logger.debug('updating session_id %s session_name %s' % (
+                    s.get('session_id'), s.get('session_name')))
                 s.update(diff[s.get('session_id')])
 
         # create session objects for non-existant session_id's
         for session in [new[session_id] for session_id in created]:
-            logging.debug('new session %s' % session['session_id'])
+            logger.debug('new session %s' % session['session_id'])
             new_session = Session(server=self, **session)
             self._sessions.append(new_session)
 
         return self._sessions
+    list_children = list_sessions
 
     def list_clients(self):
         '''
@@ -143,16 +143,16 @@ class Server(object):
         '''
         formats = CLIENT_FORMATS
         tmux_formats = ['#{%s}' % format for format in formats]
-        #import ipdb
-        #ipdb.set_trace()
+        # import ipdb
+        # ipdb.set_trace()
         clients = self.tmux(
             'list-clients',
             '-F%s' % '\t'.join(tmux_formats),   # output
-            _iter=False                          # iterate line by line
-        )
+        ).stdout
 
         # combine format keys with values returned from ``tmux list-windows``
-        clients = [dict(zip(formats, client.split('\t'))) for client in clients]
+        clients = [dict(zip(
+            formats, client.split('\t'))) for client in clients]
 
         # clear up empty dict
         new_clients = [
@@ -161,7 +161,7 @@ class Server(object):
 
         if not self._clients:
             for client in new_clients:
-                logging.debug('adding client_tty %s' % (client['client_tty']))
+                logger.debug('adding client_tty %s' % (client['client_tty']))
                 self._clients.append(client)
             return self._clients
 
@@ -172,9 +172,10 @@ class Server(object):
         deleted = set(old.keys()) - set(new.keys())
         intersect = set(new.keys()).intersection(set(old.keys()))
 
-        diff = {id: dict(set(new[id].items()) - set(old[id].items())) for id in intersect}
+        diff = {id: dict(set(new[id].items()) - set(old[id].items()))
+                for id in intersect}
 
-        logging.info(
+        logger.debug(
             "syncing clients"
             "\n\tdiff: %s\n"
             "\tcreated: %s\n"
@@ -185,16 +186,16 @@ class Server(object):
         for s in self._clients:
             # remove client objects if deleted or out of client
             if s.get('client_tty') in deleted:
-                logging.info("removing %s" % s)
+                logger.debug("removing %s" % s)
                 self._clients.remove(s)
 
             if s.get('client_tty') in intersect:
-                logging.debug('updating client_tty %s' % (s.get('client_tty')))
+                logger.debug('updating client_tty %s' % (s.get('client_tty')))
                 s.update(diff[s.get('client_tty')])
 
         # create client objects for non-existant client_tty's
         for client in [new[client_tty] for client_tty in created]:
-            logging.debug('new client %s' % client['client_tty'])
+            logger.debug('new client %s' % client['client_tty'])
             self._clients.append(client)
 
         return self._clients
@@ -208,7 +209,7 @@ class Server(object):
             self.tmux('list-clients')
             self.tmux('list-sessions')
             return True
-        except TmuxNotRunning:
+        except Exception:
             return False
 
     def has_clients(self):
@@ -217,7 +218,7 @@ class Server(object):
             return True
         else:
             return False
-        #if e.stderr == 'failed to connect to server':
+        # if e.stderr == 'failed to connect to server':
         #    raise TmuxNotRunning('tmux session not running. please start'
         #                            'a tmux session in another terminal '
         #                            'window and continue.')
@@ -236,7 +237,8 @@ class Server(object):
             if 'session_attached' in session:
                 # for now session_active is a unicode
                 if session.get('session_attached') == '1':
-                    logging.info('session %s attached', session.get('session_name'))
+                    logger.debug('session %s attached', session.get(
+                        'session_name'))
                     attached_sessions.append(session)
                 else:
                     continue
@@ -252,13 +254,14 @@ class Server(object):
         returns True if session exists.
         '''
 
-        try:  # has-session returns nothing if session exists
-            if 'session not found' in self.tmux('has-session', '-t', target_session).stderr:
-                return False
-            else:
-                return True
-        except Exception as e:
+        proc = self.tmux('has-session', '-t%s' % target_session)
+
+        if 'failed to connect to server' in proc.stdout:
             return False
+        elif 'session not found' in proc.stdout:
+            return False
+        else:
+            return True
 
     def kill_server(self):
         '''
@@ -273,7 +276,7 @@ class Server(object):
         :param: target_session: str. note this accepts fnmatch(3). 'asdf' will
                                 kill asdfasd
         '''
-        self.tmux('kill-session', '-t', target_session)
+        self.tmux('kill-session', '-t%s' % target_session)
         self.list_sessions()
 
     @property
@@ -286,8 +289,8 @@ class Server(object):
 
         :param: target_session: str. name of the session. fnmatch(3) works.
         '''
-        #tmux('switch-client', '-t', target_session)
-        self.tmux('switch-client', '-t%s' %target_session)
+        # tmux('switch-client', '-t', target_session)
+        self.tmux('switch-client', '-t%s' % target_session)
 
     def new_session(self,
                     session_name=None,
@@ -326,15 +329,16 @@ class Server(object):
         :type kill_session: bool
         '''
 
-        ### ToDo: Update below to work with attach_if_exists
+        # ToDo: Update below to work with attach_if_exists
         if self.has_session(session_name):
             if kill_session:
-                self.tmux('kill-session', '-t', session_name)
-                logging.error('session %s exists. killed it.' % session_name)
+                self.tmux('kill-session', '-t%s' % session_name)
+                logger.error('session %s exists. killed it.' % session_name)
             else:
-                raise TmuxSessionExists('Session named %s exists' % session_name)
+                raise TmuxSessionExists(
+                    'Session named %s exists' % session_name)
 
-        logging.debug('creating session %s' % session_name)
+        logger.debug('creating session %s' % session_name)
 
         formats = SESSION_FORMATS
         tmux_formats = ['#{%s}' % format for format in formats]
@@ -347,10 +351,12 @@ class Server(object):
         session_info = self.tmux(
             'new-session',
             '-d',  # assume detach = True for now, todo: fix
-            '-s', session_name,
+            '-s%s' % session_name,
             '-P', '-F%s' % '\t'.join(tmux_formats),   # output
         )
 
+        session_info = session_info.stdout[0]
+
         if env:
             os.environ['TMUX'] = env
 
